@@ -1,80 +1,83 @@
 ;;; url-gw.el --- Gateway munging for URL loading
-;; Author: wmperry
-;; Created: 1998/12/18 02:19:33
-;; Version: 1.1.1.2
+
+;; Copyright (C) 1997-1998, 2004-2012  Free Software Foundation, Inc.
+
+;; Author: Bill Perry <wmperry@gnu.org>
 ;; Keywords: comm, data, processes
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Copyright (c) 1997, 1998 Free Software Foundation, Inc.
-;;;
-;;; This file is part of GNU Emacs.
-;;;
-;;; GNU Emacs is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
-;;;
-;;; GNU Emacs is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;;; Boston, MA 02111-1307, USA.
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-(require 'cl)
+;; This file is part of GNU Emacs.
+
+;; GNU Emacs is free software: you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
+
+;; GNU Emacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+
+;;; Code:
+
+(eval-when-compile (require 'cl))
+(require 'url-vars)
+
+;; Fixme: support SSH explicitly or via a url-gateway-rlogin-program?
+
+(autoload 'socks-open-network-stream "socks")
 
 (defgroup url-gateway nil
-  "URL gateway variables"
+  "URL gateway variables."
   :group 'url)
 
 (defcustom url-gateway-local-host-regexp nil
-  "*A regular expression specifying local hostnames/machines."
+  "A regular expression specifying local hostnames/machines."
   :type '(choice (const nil) regexp)
   :group 'url-gateway)
 
 (defcustom url-gateway-prompt-pattern
   "^[^#$%>;]*[#$%>;] *" ;; "bash\\|\$ *\r?$\\|> *\r?"
-  "*A regular expression matching a shell prompt."
+  "A regular expression matching a shell prompt."
   :type 'regexp
   :group 'url-gateway)
 
 (defcustom url-gateway-rlogin-host nil
-  "*What hostname to actually rlog into before doing a telnet."
+  "What hostname to actually rlog into before doing a telnet."
   :type '(choice (const nil) string)
   :group 'url-gateway)
 
 (defcustom url-gateway-rlogin-user-name nil
-  "*Username to log into the remote machine with when using rlogin."
+  "Username to log into the remote machine with when using rlogin."
   :type '(choice (const nil) string)
   :group 'url-gateway)
 
 (defcustom url-gateway-rlogin-parameters '("telnet" "-8")
-  "*Parameters to `url-open-rlogin'.
+  "Parameters to `url-open-rlogin'.
 This list will be used as the parameter list given to rsh."
   :type '(repeat string)
   :group 'url-gateway)
 
 (defcustom url-gateway-telnet-host nil
-  "*What hostname to actually login to before doing a telnet."
+  "What hostname to actually login to before doing a telnet."
   :type '(choice (const nil) string)
   :group 'url-gateway)
 
 (defcustom url-gateway-telnet-parameters '("exec" "telnet" "-8")
-  "*Parameters to `url-open-telnet'.
+  "Parameters to `url-open-telnet'.
 This list will be executed as a command after logging in via telnet."
   :type '(repeat string)
   :group 'url-gateway)
 
 (defcustom url-gateway-telnet-login-prompt "^\r*.?login:"
-  "*Prompt that tells us we should send our username when logging in w/telnet."
+  "Prompt that tells us we should send our username when logging in w/telnet."
   :type 'regexp
   :group 'url-gateway)
 
 (defcustom url-gateway-telnet-password-prompt "^\r*.?password:"
-  "*Prompt that tells us we should send our password when logging in w/telnet."
+  "Prompt that tells us we should send our password when logging in w/telnet."
   :type 'regexp
   :group 'url-gateway)
 
@@ -89,30 +92,31 @@ This list will be executed as a command after logging in via telnet."
   :group 'url-gateway)
 
 (defcustom url-gateway-broken-resolution nil
-  "*Whether to use nslookup to resolve hostnames.
+  "Whether to use nslookup to resolve hostnames.
 This should be used when your version of Emacs cannot correctly use DNS,
 but your machine can.  This usually happens if you are running a statically
-linked Emacs under SunOS 4.x"
+linked Emacs under SunOS 4.x."
   :type 'boolean
   :group 'url-gateway)
 
 (defcustom url-gateway-nslookup-program "nslookup"
-  "*If non-NIL then a string naming nslookup program."
+  "If non-nil then a string naming nslookup program."
   :type '(choice (const :tag "None" :value nil) string)
   :group 'url-gateway)
 
 ;; Stolen from ange-ftp
 ;;;###autoload
 (defun url-gateway-nslookup-host (host)
-  "Attempt to resolve the given HOSTNAME using nslookup if possible."
+  "Attempt to resolve the given HOST using nslookup if possible."
   (interactive "sHost:  ")
   (if url-gateway-nslookup-program
       (let ((proc (start-process " *nslookup*" " *nslookup*"
 				 url-gateway-nslookup-program host))
 	    (res host))
-	(process-kill-without-query proc)
-	(save-excursion
-	  (set-buffer (process-buffer proc))
+        (if (featurep 'xemacs)
+            (process-kill-without-query proc t)
+          (set-process-query-on-exit-flag proc nil))
+	(with-current-buffer (process-buffer proc)
 	  (while (memq (process-status proc) '(run open))
 	    (accept-process-output proc))
 	  (goto-char (point-min))
@@ -125,7 +129,7 @@ linked Emacs under SunOS 4.x"
 
 ;; Stolen from red gnus nntp.el
 (defun url-wait-for-string (regexp proc)
-  "Wait until string arrives in the buffer."
+  "Wait until string matching REGEXP arrives in process PROC's buffer."
   (let ((buf (current-buffer)))
     (goto-char (point-min))
     (while (not (re-search-forward regexp nil t))
@@ -161,8 +165,7 @@ linked Emacs under SunOS 4.x"
 (defun url-open-telnet (name buffer host service)
   (if (not (stringp service))
       (setq service (int-to-string service)))
-  (save-excursion
-    (set-buffer (get-buffer-create buffer))
+  (with-current-buffer (get-buffer-create buffer)
     (erase-buffer)
     (let ((proc (start-process name buffer "telnet" "-8"))
 	  (case-fold-search t))
@@ -181,7 +184,7 @@ linked Emacs under SunOS 4.x"
 	 proc (concat
 	       (or url-gateway-telnet-password
 		   (setq url-gateway-telnet-password
-			 (funcall url-passwd-entry-func "Password: ")))
+			 (read-passwd "Password: ")))
 	       "\n"))
 	(erase-buffer)
 	(url-wait-for-string url-gateway-prompt-pattern proc)
@@ -203,57 +206,61 @@ linked Emacs under SunOS 4.x"
 
 ;;;###autoload
 (defun url-open-stream (name buffer host service)
-  "Open a stream to a host"
-  (let ((gw-method (if (and url-gateway-local-host-regexp
-			    (not (eq 'ssl url-gateway-method))
-			    (string-match
-			     url-gateway-local-host-regexp
-			     host))
-		       'native
-		     url-gateway-method))
-	;; This hack is for OS/2 Emacs so that it will not do bogus CRLF
-	;; conversions while trying to be 'helpful'
-	(tcp-binary-process-output-services (if (stringp service)
-						(list service)
-					      (list service
-						    (int-to-string service))))
+  "Open a stream to HOST, possibly via a gateway.
+Args per `open-network-stream'.
+Will not make a connection if `url-gateway-unplugged' is non-nil.
+Might do a non-blocking connection; use `process-status' to check."
+  (unless url-gateway-unplugged
+    (let ((gw-method (if (and url-gateway-local-host-regexp
+			      (not (eq 'tls url-gateway-method))
+			      (not (eq 'ssl url-gateway-method))
+			      (string-match
+			       url-gateway-local-host-regexp
+			       host))
+			 'native
+		       url-gateway-method))
+	  ;; An attempt to deal with denied connections, and attempt
+	  ;; to reconnect
+	  (cur-retries 0)
+	  (retry t)
+	  (errobj nil)
+	  (conn nil))
 
-	;; An attempt to deal with denied connections, and attempt to reconnect
-	(cur-retries 0)
-	(retry t)
-	(errobj nil)
-	(conn nil))
+      ;; If the user told us to do DNS for them, do it.
+      (if url-gateway-broken-resolution
+	  (setq host (url-gateway-nslookup-host host)))
 
-    ;; If the user told us to do DNS for them, do it.
-    (if url-gateway-broken-resolution
-	(setq host (url-gateway-nslookup-host host)))
-
-    (condition-case errobj
-	(setq conn (case gw-method
-		     (ssl
-		      (open-ssl-stream name buffer host service))
-		     ((tcp native)
-		      (and (eq 'tcp gw-method) (require 'tcp))
-		      (open-network-stream name buffer host service))
-		     (socks
-		      (socks-open-network-stream name buffer host service))
-		     (telnet
-		      (url-open-telnet name buffer host service))
-		     (rlogin
-		      (url-open-rlogin name buffer host service))
-		     (otherwise
-		      (error "Bad setting of url-gateway-method: %s"
-			     url-gateway-method))))
-      (error
-       (insert "Could not contact host: " host " / "
-	       (if (stringp service) service (int-to-string service))
-	       "\nAttempted using gateway method: "
-	       (symbol-name gw-method)
-	       "\n---- Error was: ----\n")
-       (setq url-current-mime-headers '(("content-type" . "text/plain")))
-       (display-error errobj (current-buffer))))
-    (if conn
-	(mule-inhibit-code-conversion conn))
-    conn))
+      (condition-case errobj
+	  ;; This is a clean way to ensure the new process inherits the
+	  ;; right coding systems in both Emacs and XEmacs.
+	  (let ((coding-system-for-read 'binary)
+		(coding-system-for-write 'binary))
+	    (setq conn (case gw-method
+			 ((tls ssl native)
+			  (if (eq gw-method 'native)
+			      (setq gw-method 'plain))
+			  (apply
+                           #'open-network-stream
+			   name buffer host service
+                           (if (or (not (featurep 'xemacs))
+                                   (function-allows-args 'open-network-stream
+                                                         8))
+                             `(:type gw-method
+                               ;; Use non-blocking socket if we can.
+                               :nowait ,(ignore-errors
+                                          (featurep 'make-network-process
+                                                    '(:nowait t)))))))
+			 (socks
+			  (socks-open-network-stream name buffer host service))
+			 (telnet
+			  (url-open-telnet name buffer host service))
+			 (rlogin
+			  (url-open-rlogin name buffer host service))
+			 (otherwise
+			  (error "Bad setting of url-gateway-method: %s"
+				 url-gateway-method))))))
+      conn)))
 
 (provide 'url-gw)
+
+;;; url-gw.el ends here
