@@ -26,7 +26,16 @@
 (require 'url-vars)
 (require 'url-parse)
 (require 'url-util)
-(require 'ldap)
+;; We need the LDAP module available when run interpreted and when run
+;; compiled, but not at byte-compile time. Don't force the XEmacs building
+;; this to have support for the LDAP module, that's not needed for
+;; byte-compiling Lisp.
+(eval-when (:load-toplevel :execute)
+  ;; Use a contorted syntax for the require call so its byte-hunk-handler
+  ;; isn't called, but it's still compiled to the usual code.
+  ;; I plan to update the byte compiler to make the eval-when sufficient going
+  ;; forward. Aidan Kehoe, Mo 11 Jan 2021 09:52:40 GMT
+  (apply #'require 'ldap nil))
 (autoload 'tls-certificate-information "tls")
 
 ;; This has been implemented from RFC2255 'The LDAP URL Format' (Dec 1997)
@@ -189,13 +198,24 @@ URL can be a URL string, or a URL vector of the type returned by
 				   (assoc "!bindname" extensions))))
 
 	;; Now, let's actually do something with it.
-	(setq results (cdr (ldap-search-internal
-		       (list 'host (concat host ":" (number-to-string port))
-			     'base base-object
-			     'attributes attributes
-			     'scope scope
-			     'filter filter
-			     'binddn binddn))))
+        (let* ((this-host-alist (cdr (assoc (if port
+                                                (format "%s:%d" host port)
+                                              host)
+                                            ldap-host-parameters-alist)))
+               (ldap-host-parameters-alist
+                (cons `(,host port ,(or port
+                                        (and (assq 'port this-host-alist)
+                                             (int-to-string
+                                              (cdr (assq 'port
+                                                         this-host-alist))))
+                                        ldap-default-port)
+                              base ,base-object scope ,scope binddn ,binddn
+                              ,@this-host-alist)
+                      ldap-host-parameters-alist)))
+          (setq results
+                ;; Use #'ldap-search rather than our newer
+                ;; #'ldap-search-entries to attempt portability to GNU.
+                (ldap-search filter host attributes)))
 
 	(insert "<html>\n"
 		" <head>\n"
