@@ -1,7 +1,7 @@
 ;;; w3-hot.el --- Main functions for emacs-w3 on all platforms/versions
-;; Author: wmperry
-;; Created: 1998/12/18 02:19:48
-;; Version: 1.1.1.2
+;; Author: $Author: fx $
+;; Created: $Date: 2001/05/14 15:51:54 $
+;; Version: $Revision: 1.4 $
 ;; Keywords: faces, help, comm, news, mail, processes, mouse, hypermedia
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -37,32 +37,15 @@
 ;;; )  ; end of hotlist
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 (require 'w3-vars)
+(require 'w3-parse)
+(require 'url-parse)
+(eval-when-compile (require 'cl))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Hotlist Handling Code
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 (defvar w3-html-bookmarks nil)
 
-(defun w3-hotlist-break-shit ()
-  (let ((todo '(w3-hotlist-apropos
-		w3-hotlist-delete
-		w3-hotlist-rename-entry
-		w3-hotlist-append
-		w3-use-hotlist
-		w3-hotlist-add-document
-		w3-hotlist-add-document-at-point
-		))
-	(cur nil))
-    (while todo
-      (setq cur (car todo)
-	    todo (cdr todo))
-      (fset cur
-	    (`
-	     (lambda (&rest ignore)
-	       (interactive)
-	       (error "Sorry, `%s' does not work with html bookmarks"
-		      (quote (, cur)))))))))
-
 ;;;###autoload
 (defun w3-read-html-bookmarks (fname)
   "Import an HTML file into the Emacs-w3 format."
@@ -78,22 +61,27 @@
 	   (parse (w3-parse-buffer (current-buffer))))
       (setq parse w3-last-parse-tree
 	    bkmarks (nreverse (w3-grok-html-bookmarks parse))
-	    w3-html-bookmarks bkmarks)))
-  (w3-hotlist-break-shit))
+	    w3-hotlist bkmarks))))
 
 (eval-when-compile
-  (defvar cur-stack nil)
-  (defvar cur-title nil)
-  (defmacro push-new-menu ()
-    '(setq cur-stack (cons (list "") cur-stack)))
+  (defsubst w3-hot-push-new-menu ()
+    (declare (special cur-stack))
+    (setq cur-stack (cons (list "") cur-stack)))
+
+  ;; This stores it in menu format
+  '(defsubst w3-hot-push-new-item (title href)
+    (declare (special cur-stack))
+    (setcar cur-stack (cons (vector title (list 'w3-fetch href) t)
+			    (car cur-stack))))
+
+  ;; This stores it in alist format
+  (defsubst w3-hot-push-new-item (title href)
+    (declare (special cur-stack))
+    (setcar cur-stack (cons (cons title href) (car cur-stack))))
   
-  (defmacro push-new-item (title href)
-    (` (setcar cur-stack (cons (vector (, title) (list 'w3-fetch (, href)) t)
-			       (car cur-stack)))))
-  ;;(` (setcar cur-stack (cons (cons (, title) (, href)) (car cur-stack)))))
-  
-  (defmacro finish-submenu ()
-    '(let ((x (nreverse (car cur-stack)))
+  (defsubst w3-hot-finish-submenu ()
+    (declare (special cur-stack cur-title))
+    (let ((x (nreverse (car cur-stack)))
 	   (y (pop cur-title)))
        (while (string= y "")
 	 (setq y (pop cur-title)))
@@ -105,6 +93,7 @@
   )
 
 (defun w3-grok-html-bookmarks-internal (tree)
+  (declare (special cur-stack cur-title))
   (let (node tag content args)
     (while tree
       (setq node (car tree)
@@ -113,13 +102,15 @@
 	    args (and (listp node) (nth 1 node))
 	    content (and (listp node) (nth 2 node)))
       (cond
+       ((eq tag 'hr)
+	(setq cur-title '("------")))
        ((eq tag 'title)
 	(setq cur-title (list (w3-normalize-spaces (car content))))
 	(w3-grok-html-bookmarks-internal content))
        ((memq tag '(dl ol ul))
-	(push-new-menu)
+	(w3-hot-push-new-menu)
 	(w3-grok-html-bookmarks-internal content)
-	(finish-submenu))
+	(w3-hot-finish-submenu))
        ((and (memq tag '(dt li p))
 	     (stringp (car content)))
 	(setq cur-title (cons (w3-normalize-spaces (car content))
@@ -127,7 +118,7 @@
        ((and (eq tag 'a)
 	     (stringp (car-safe content))
 	     (cdr-safe (assq 'href args)))
-	(push-new-item (w3-normalize-spaces (car-safe content))
+	(w3-hot-push-new-item (w3-normalize-spaces (car-safe content))
 		       (cdr-safe (assq 'href args))))
        (content
 	(w3-grok-html-bookmarks-internal content))))))
@@ -137,132 +128,37 @@
 	cur-title
 	cur-stack
 	)
+    (declare (special cur-title cur-stack))
     (w3-grok-html-bookmarks-internal chunk)
     (reverse (car cur-stack))))
 
-;;;###autoload
-(defun w3-hotlist-apropos (regexp)
-  "Show hotlist entries matching REGEXP."
-  (interactive "sW3 Hotlist Apropos (regexp): ")
-  (or w3-setup-done (w3-do-setup))
-  (let ((save-buf (get-buffer "Hotlist")) ; avoid killing this
-	(w3-hotlist
-	 (apply
-	  'nconc
-	  (mapcar
-	   (function
-	    (lambda (entry)
-	      (if (or (string-match regexp (car entry))
-		      (string-match regexp (car (cdr entry))))
-		  (list entry))))
-	   w3-hotlist))))
-    (if (not w3-hotlist)
-	(message "No w3-hotlist entries match \"%s\"" regexp)
-      (and save-buf (save-excursion
-		      (set-buffer save-buf)
-		      (rename-buffer (concat "Hotlist during " regexp))))
-      (unwind-protect
-	  (let ((w3-reuse-buffers 'no))
-	    (w3-show-hotlist)
-	    (rename-buffer (concat "Hotlist \"" regexp "\""))
-	    (url-set-filename url-current-object (concat "hotlist/" regexp)))
-	(and save-buf (save-excursion
-			(set-buffer save-buf)
-			(rename-buffer "Hotlist")))))))
+(defun w3-hot-convert-to-alist-mapper (node)
+  (declare (special prefix alist))
+  (cond
+   ((stringp node)
+    ;; Top-level node... ignore
+    )
+   ((stringp (cdr node))
+    ;; A real hyperlink, push it onto the alist
+    (push (cons (if prefix (concat prefix " / " (car node)) (car node)) (cdr node)) alist))
+   (t
+    ;; A submenu, add to prefix and recurse
+    (w3-hot-convert-to-alist-internal
+     (cdr node) (if prefix (concat prefix " / " (car node)) (car node))))))
 
-;;;###autoload
-(defun w3-hotlist-refresh ()
-  "Reload the default hotlist file into memory"
-  (interactive)
-  (if (not w3-setup-done) (w3-do-setup))
-  (w3-parse-hotlist))
+(defun w3-hot-convert-to-alist-internal (l &optional prefix)
+  (mapc 'w3-hot-convert-to-alist-mapper l))
+
+(defun w3-hot-convert-to-alist (l)
+  (let ((alist nil))
+    (w3-hot-convert-to-alist-internal l)
+    alist))
 
 (defun w3-delete-from-alist (x alist)
   ;; Remove X from ALIST, return new alist
   (if (eq (assoc x alist) (car alist)) (cdr alist)
     (delq (assoc x alist) alist)))
 
-;;;###autoload
-(defun w3-hotlist-delete ()
-  "Deletes a document from your hotlist file"
-  (interactive)
-  (save-excursion
-    (if (not w3-hotlist) (message "No hotlist in memory!")
-      (if (not (file-exists-p w3-hotlist-file))
-	  (message "Hotlist file %s does not exist." w3-hotlist-file)
-	(let* ((completion-ignore-case t)
-	       (title (car (assoc (completing-read "Delete Document: "
-						   w3-hotlist nil t)
-				  w3-hotlist)))
-	       (case-fold-search nil)
-	       (buffer (get-buffer-create " *HOTW3*")))
-	  (and (string= title "") (error "No document specified."))
-	  (set-buffer buffer)
-	  (erase-buffer)
-	  (insert-file-contents w3-hotlist-file)
-	  (goto-char (point-min))
-	  (if (re-search-forward (concat "^" (regexp-quote title) "\r*$")
-				 nil t)
-	      (let ((make-backup-files nil)
-		    (version-control nil)
-		    (require-final-newline t))
-		(previous-line 1)
-		(beginning-of-line)
-		(delete-region (point) (progn (forward-line 2) (point)))
-		(write-file w3-hotlist-file)
-		(setq w3-hotlist (w3-delete-from-alist title w3-hotlist))
-		(kill-buffer (current-buffer))
-		(w3-hotindex-delete-entry title))
-	    (message "%s was not found in %s" title w3-hotlist-file)))))))
-
-;;;###autoload
-(defun w3-hotlist-rename-entry (title)
-  "Rename a hotlist item"
-  (interactive (list (let ((completion-ignore-case t))
-		       (completing-read "Rename entry: " w3-hotlist nil t))))
-  (cond					; Do the error handling first
-   ((string= title "") (error "No document specified!"))
-   ((not w3-hotlist) (error "No hotlist in memory!"))
-   ((not (file-exists-p (expand-file-name w3-hotlist-file)))
-    (error "Hotlist file %s does not exist." w3-hotlist-file))
-   ((not (file-readable-p (expand-file-name w3-hotlist-file)))
-    (error "Hotlist file %s exists, but is unreadable." w3-hotlist-file)))
-  (save-excursion
-    (let ((obj (assoc title w3-hotlist))
-	  (used (mapcar 'car w3-hotlist))
-	  (buff (get-buffer-create " *HOTW3*"))
-	  (new nil)
-	  )
-      (while (or (null new) (member new used))
-	(setq new (read-string "New name: ")))
-      (set-buffer buff)
-      (erase-buffer)
-      (insert-file-contents (expand-file-name w3-hotlist-file))
-      (goto-char (point-min))
-      (if (re-search-forward (concat "^" (regexp-quote title) "$") nil t)
-	  (let ((make-backup-files nil)
-		(version-control nil)
-		(require-final-newline t))
-	    (previous-line 1)
-	    (beginning-of-line)
-	    (delete-region (point) (progn (forward-line 2) (point)))
-	    (insert (format "%s %s\n%s\n" (nth 1 obj) (current-time-string)
-			    new))
-	    (setq w3-hotlist (cons (list new (nth 1 obj))
-				   (w3-delete-from-alist title w3-hotlist)))
-	    (write-file w3-hotlist-file)
- 	    (w3-hotindex-rename-entry title new)
-	    (kill-buffer (current-buffer)))
-	(message "%s was not found in %s" title w3-hotlist-file)))))
-
-;;;###autoload
-(defun w3-hotlist-append (fname)
-  "Append a hotlist to the one in memory"
-  (interactive "fAppend hotlist file: ")
-  (let ((x w3-hotlist))
-    (w3-parse-hotlist fname)
-    (setq w3-hotlist (nconc x w3-hotlist))))
-
 (defun w3-hotlist-parse-old-mosaic-format ()
   (let (cur-link cur-alias)
     (while (re-search-forward "^\n" nil t) (replace-match ""))
@@ -316,9 +212,10 @@ visited or interesting items you have found on the World Wide Web."
   (if (not w3-setup-done) (w3-do-setup))
   (if (not w3-hotlist) (message "No hotlist in memory!")
     (let* ((completion-ignore-case t)
-	   (url (car (cdr (assoc
-			   (completing-read "Goto Document: " w3-hotlist nil t)
-			   w3-hotlist)))))
+	   (hot-alist (w3-hot-convert-to-alist w3-hotlist))
+	   (url (cdr (assoc
+		      (completing-read "Goto Document: " hot-alist nil t)
+		      hot-alist))))
       (if (string= "" url) (error "No document specified!"))
       (w3-fetch url))))
 
@@ -340,32 +237,32 @@ visited or interesting items you have found on the World Wide Web."
 (defun w3-hotlist-add-document (pref-arg &optional the-title the-url)
   "Add this documents url to the hotlist"
   (interactive "P")
-  (save-excursion
-    (let* ((buffer (get-buffer-create " *HOTW3*"))
-	   (title (or the-title
-		      (and pref-arg (read-string "Title: "))
-		      (buffer-name)))
-	   (make-backup-files nil)
-	   (version-control nil)
-	   (require-final-newline t)
-	   (url (or the-url (url-view-url t))))
-      (if (rassoc (list url) w3-hotlist)
-	  (error "That item already in hotlist, use w3-hotlist-rename-entry."))
-      (set-buffer buffer)
-      (erase-buffer)
-      (setq w3-hotlist (cons (list title url) w3-hotlist)
-	    url (url-unhex-string url))
-      (if (not (file-exists-p w3-hotlist-file))
-	  (progn
-	    (message "Creating hotlist file %s" w3-hotlist-file)
-	    (insert "ncsa-xmosaic-hotlist-format-1\nDefault\n\n")
-	    (backward-char 1))
-	(progn
-	  (insert-file-contents w3-hotlist-file)
-	  (goto-char (point-max))
-	  (backward-char 1)))
-      (insert "\n" url " " (current-time-string) "\n" title)
-      (write-file w3-hotlist-file)
-      (kill-buffer (current-buffer)))))
+  (error "Adding to hotlist not implemented yet."))
+
+;;;###autoload
+(defun w3-hotlist-delete ()
+  "Deletes a document from your hotlist file"
+  (interactive)
+  (error "Deleting from hotlist not implemented yet."))
+
+;;;###autoload
+(defun w3-hotlist-refresh ()
+  "Reload the default hotlist file into memory"
+  (interactive)
+  (w3-do-setup)
+  (w3-parse-hotlist))
+
+;;;###autoload
+(defun w3-hotlist-apropos (regexp)
+  "Show hotlist entries matching REGEXP."
+  (interactive "sW3 Hotlist Apropos (regexp): ")
+  (or w3-setup-done (w3-do-setup))
+  (w3-fetch (concat "hotlist:search?regexp=" (url-hexify-string regexp))))
+
+;;;###autoload
+(defun w3-hotlist-view ()
+  "Show the hotlist."
+  (interactive)
+  (w3-fetch "hotlist:view"))
 
 (provide 'w3-hot)
