@@ -1,7 +1,7 @@
 ;;; w3-style.el --- Emacs/W3 binding style sheet mechanism
-;; Author: wmperry
-;; Created: 1998/12/18 02:19:51
-;; Version: 1.1.1.2
+;; Author: $Author: fx $
+;; Created: $Date: 2001/10/11 13:04:58 $
+;; Version: $Revision: 1.4 $
 ;; Keywords: faces, hypermedia
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -33,12 +33,9 @@
 ;;; experimental W3C mechanism
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 (require 'font)
-(require 'w3-keyword)
-(require 'cl)
+(eval-when-compile (require 'cl))
 (require 'css)
 
-
-
 ;;;###autoload
 (defun w3-handle-style (&optional plist)
   (let ((url (or (plist-get plist 'href)
@@ -46,17 +43,14 @@
 		 (plist-get plist 'uri)))
 	(media (intern (downcase (or (plist-get plist 'media) "all"))))
 	(type (downcase (or (plist-get plist 'notation) "text/css")))
-	(url-working-buffer " *style*")
 	(stylesheet nil)
 	(defines nil)
+	(buffer nil)
 	(cur-sheet w3-current-stylesheet)
 	(string (plist-get plist 'data)))
     (if (not (memq media (css-active-device-types)))
 	nil				; Not applicable to us!
       (save-excursion
-	(set-buffer (get-buffer-create url-working-buffer))
-	(erase-buffer)
-	(setq url-be-asynchronous nil)
 	(cond
 	 ((member type '("experimental" "arena" "w3c-style" "css" "text/css"))
 	  (setq stylesheet (css-parse url string cur-sheet)))
